@@ -25,7 +25,7 @@
 mod client;
 pub use client::HttpClient;
 pub use client::HttpFetch;
-
+pub use client::HttpFetcher;
 /// temporary client used by several features
 #[allow(unused_imports)]
 pub(crate) use client::GLOBAL_REQWEST_CLIENT;
@@ -56,6 +56,7 @@ mod uri;
 pub use uri::new_http_uri_invalid_error;
 pub use uri::percent_decode_path;
 pub use uri::percent_encode_path;
+pub use uri::QueryPairsWriter;
 
 mod error;
 pub use error::new_request_build_error;
@@ -74,3 +75,4 @@ pub use multipart::FormDataPart;
 pub use multipart::MixedPart;
 pub use multipart::Multipart;
 pub use multipart::Part;
+pub use multipart::RelatedPart;
