@@ -15,11 +15,22 @@
 // specific language governing permissions and limitations
 // under the License.
 
-mod backend;
-pub use backend::AlluxioBuilder as Alluxio;
-pub use backend::AlluxioConfig;
-
+#[cfg(feature = "services-alluxio")]
 mod core;
+#[cfg(feature = "services-alluxio")]
+mod delete;
+#[cfg(feature = "services-alluxio")]
 mod error;
+#[cfg(feature = "services-alluxio")]
 mod lister;
+#[cfg(feature = "services-alluxio")]
 mod writer;
+
+#[cfg(feature = "services-alluxio")]
+mod backend;
+#[cfg(feature = "services-alluxio")]
+pub use backend::AlluxioBuilder as Alluxio;
+
+mod config;
+
+pub use config::AlluxioConfig;
