@@ -15,7 +15,6 @@
 // specific language governing permissions and limitations
 // under the License.
 
-use std::collections::HashMap;
 use std::fmt::Debug;
 use std::fmt::Formatter;
 use std::sync::Arc;
@@ -32,6 +31,7 @@ use sha2::Digest;
 use sha2::Sha256;
 
 use super::core::constants::X_MS_META_PREFIX;
+use super::core::constants::X_MS_VERSION_ID;
 use super::core::AzblobCore;
 use super::delete::AzblobDeleter;
 use super::error::parse_error;
@@ -42,23 +42,28 @@ use crate::raw::*;
 use crate::services::AzblobConfig;
 use crate::*;
 
-/// Known endpoint suffix Azure Storage Blob services resource URI syntax.
-/// Azure public cloud: https://accountname.blob.core.windows.net
-/// Azure US Government: https://accountname.blob.core.usgovcloudapi.net
-/// Azure China: https://accountname.blob.core.chinacloudapi.cn
-const KNOWN_AZBLOB_ENDPOINT_SUFFIX: &[&str] = &[
-    "blob.core.windows.net",
-    "blob.core.usgovcloudapi.net",
-    "blob.core.chinacloudapi.cn",
-];
-
 const AZBLOB_BATCH_LIMIT: usize = 256;
 
+impl From<AzureStorageConfig> for AzblobConfig {
+    fn from(value: AzureStorageConfig) -> Self {
+        Self {
+            endpoint: value.endpoint,
+            account_name: value.account_name,
+            account_key: value.account_key,
+            sas_token: value.sas_token,
+            ..Default::default()
+        }
+    }
+}
+
 impl Configurator for AzblobConfig {
     type Builder = AzblobBuilder;
+
+    #[allow(deprecated)]
     fn into_builder(self) -> Self::Builder {
         AzblobBuilder {
             config: self,
+
             http_client: None,
         }
     }
@@ -68,6 +73,8 @@ impl Configurator for AzblobConfig {
 #[derive(Default, Clone)]
 pub struct AzblobBuilder {
     config: AzblobConfig,
+
+    #[deprecated(since = "0.53.0", note = "Use `Operator::update_http_client` instead")]
     http_client: Option<HttpClient>,
 }
 
@@ -244,6 +251,8 @@ impl AzblobBuilder {
     ///
     /// This API is part of OpenDAL's Raw API. `HttpClient` could be changed
     /// during minor updates.
+    #[deprecated(since = "0.53.0", note = "Use `Operator::update_http_client` instead")]
+    #[allow(deprecated)]
     pub fn http_client(mut self, client: HttpClient) -> Self {
         self.http_client = Some(client);
         self
@@ -281,64 +290,13 @@ impl AzblobBuilder {
     ///
     /// # Note
     ///
-    /// connection string only configures the endpoint, account name and account key.
-    /// User still needs to configure bucket names.
+    /// Connection strings can only configure the endpoint, account name and
+    /// authentication information. Users still need to configure container name.
     pub fn from_connection_string(conn: &str) -> Result<Self> {
-        let conn = conn.trim().replace('\n', "");
+        let config =
+            raw::azure_config_from_connection_string(conn, raw::AzureStorageService::Blob)?;
 
-        let mut conn_map: HashMap<_, _> = HashMap::default();
-        for v in conn.split(';') {
-            let entry: Vec<_> = v.splitn(2, '=').collect();
-            if entry.len() != 2 {
-                // Ignore invalid entries.
-                continue;
-            }
-            conn_map.insert(entry[0], entry[1]);
-        }
-
-        let mut builder = AzblobBuilder::default();
-
-        if let Some(sas_token) = conn_map.get("SharedAccessSignature") {
-            builder = builder.sas_token(sas_token);
-        } else {
-            let account_name = conn_map.get("AccountName").ok_or_else(|| {
-                Error::new(
-                    ErrorKind::ConfigInvalid,
-                    "connection string must have AccountName",
-                )
-                .with_operation("Builder::from_connection_string")
-            })?;
-            builder = builder.account_name(account_name);
-            let account_key = conn_map.get("AccountKey").ok_or_else(|| {
-                Error::new(
-                    ErrorKind::ConfigInvalid,
-                    "connection string must have AccountKey",
-                )
-                .with_operation("Builder::from_connection_string")
-            })?;
-            builder = builder.account_key(account_key);
-        }
-
-        if let Some(v) = conn_map.get("BlobEndpoint") {
-            builder = builder.endpoint(v);
-        } else if let Some(v) = conn_map.get("EndpointSuffix") {
-            let protocol = conn_map.get("DefaultEndpointsProtocol").unwrap_or(&"https");
-            let account_name = builder
-                .config
-                .account_name
-                .as_ref()
-                .ok_or_else(|| {
-                    Error::new(
-                        ErrorKind::ConfigInvalid,
-                        "connection string must have AccountName",
-                    )
-                    .with_operation("Builder::from_connection_string")
-                })?
-                .clone();
-            builder = builder.endpoint(&format!("{protocol}://{account_name}.blob.{v}"));
-        }
-
-        Ok(builder)
+        Ok(AzblobConfig::from(config).into_builder())
     }
 }
 
@@ -350,7 +308,7 @@ impl Builder for AzblobBuilder {
         debug!("backend build started: {:?}", &self);
 
         let root = normalize_root(&self.config.root.unwrap_or_default());
-        debug!("backend use root {}", root);
+        debug!("backend use root {root}");
 
         // Handle endpoint, region and container name.
         let container = match self.config.container.is_empty() {
@@ -369,22 +327,13 @@ impl Builder for AzblobBuilder {
         }?;
         debug!("backend use endpoint {}", &container);
 
-        let client = if let Some(client) = self.http_client {
-            client
-        } else {
-            HttpClient::new().map_err(|err| {
-                err.with_operation("Builder::build")
-                    .with_context("service", Scheme::Azblob)
-            })?
-        };
-
         let mut config_loader = AzureStorageConfig::default().from_env();
 
         if let Some(v) = self
             .config
             .account_name
             .clone()
-            .or_else(|| infer_storage_name_from_endpoint(endpoint.as_str()))
+            .or_else(|| raw::azure_account_name_from_endpoint(endpoint.as_str()))
         {
             config_loader.account_name = Some(v);
         }
@@ -434,6 +383,60 @@ impl Builder for AzblobBuilder {
 
         Ok(AzblobBackend {
             core: Arc::new(AzblobCore {
+                info: {
+                    let am = AccessorInfo::default();
+                    am.set_scheme(Scheme::Azblob)
+                        .set_root(&root)
+                        .set_name(container)
+                        .set_native_capability(Capability {
+                            stat: true,
+                            stat_with_if_match: true,
+                            stat_with_if_none_match: true,
+
+                            read: true,
+
+                            read_with_if_match: true,
+                            read_with_if_none_match: true,
+                            read_with_override_content_disposition: true,
+                            read_with_if_modified_since: true,
+                            read_with_if_unmodified_since: true,
+
+                            write: true,
+                            write_can_append: true,
+                            write_can_empty: true,
+                            write_can_multi: true,
+                            write_with_cache_control: true,
+                            write_with_content_type: true,
+                            write_with_if_not_exists: true,
+                            write_with_if_none_match: true,
+                            write_with_user_metadata: true,
+
+                            delete: true,
+                            delete_max_size: Some(AZBLOB_BATCH_LIMIT),
+
+                            copy: true,
+
+                            list: true,
+                            list_with_recursive: true,
+
+                            presign: self.config.sas_token.is_some(),
+                            presign_stat: self.config.sas_token.is_some(),
+                            presign_read: self.config.sas_token.is_some(),
+                            presign_write: self.config.sas_token.is_some(),
+
+                            shared: true,
+
+                            ..Default::default()
+                        });
+
+                    // allow deprecated api here for compatibility
+                    #[allow(deprecated)]
+                    if let Some(client) = self.http_client {
+                        am.update_http_client(|_| client);
+                    }
+
+                    am.into()
+                },
                 root,
                 endpoint,
                 encryption_key,
@@ -441,44 +444,17 @@ impl Builder for AzblobBuilder {
                 encryption_algorithm,
                 container: self.config.container.clone(),
 
-                client,
                 loader: cred_loader,
                 signer,
             }),
-            has_sas_token: self.config.sas_token.is_some(),
         })
     }
 }
 
-fn infer_storage_name_from_endpoint(endpoint: &str) -> Option<String> {
-    let endpoint: &str = endpoint
-        .strip_prefix("http://")
-        .or_else(|| endpoint.strip_prefix("https://"))
-        .unwrap_or(endpoint);
-
-    let mut parts = endpoint.splitn(2, '.');
-    let storage_name = parts.next();
-    let endpoint_suffix = parts
-        .next()
-        .unwrap_or_default()
-        .trim_end_matches('/')
-        .to_lowercase();
-
-    if KNOWN_AZBLOB_ENDPOINT_SUFFIX
-        .iter()
-        .any(|s| *s == endpoint_suffix.as_str())
-    {
-        storage_name.map(|s| s.to_string())
-    } else {
-        None
-    }
-}
-
 /// Backend for azblob services.
 #[derive(Debug, Clone)]
 pub struct AzblobBackend {
     core: Arc<AzblobCore>,
-    has_sas_token: bool,
 }
 
 impl Access for AzblobBackend {
@@ -486,72 +462,9 @@ impl Access for AzblobBackend {
     type Writer = AzblobWriters;
     type Lister = oio::PageLister<AzblobLister>;
     type Deleter = oio::BatchDeleter<AzblobDeleter>;
-    type BlockingReader = ();
-    type BlockingWriter = ();
-    type BlockingLister = ();
-    type BlockingDeleter = ();
 
     fn info(&self) -> Arc<AccessorInfo> {
-        let mut am = AccessorInfo::default();
-        am.set_scheme(Scheme::Azblob)
-            .set_root(&self.core.root)
-            .set_name(&self.core.container)
-            .set_native_capability(Capability {
-                stat: true,
-                stat_with_if_match: true,
-                stat_with_if_none_match: true,
-                stat_has_cache_control: true,
-                stat_has_content_length: true,
-                stat_has_content_type: true,
-                stat_has_content_encoding: true,
-                stat_has_content_range: true,
-                stat_has_etag: true,
-                stat_has_content_md5: true,
-                stat_has_last_modified: true,
-                stat_has_content_disposition: true,
-
-                read: true,
-
-                read_with_if_match: true,
-                read_with_if_none_match: true,
-                read_with_override_content_disposition: true,
-                read_with_if_modified_since: true,
-                read_with_if_unmodified_since: true,
-
-                write: true,
-                write_can_append: true,
-                write_can_empty: true,
-                write_can_multi: true,
-                write_with_cache_control: true,
-                write_with_content_type: true,
-                write_with_if_not_exists: true,
-                write_with_if_none_match: true,
-                write_with_user_metadata: true,
-
-                delete: true,
-                delete_max_size: Some(AZBLOB_BATCH_LIMIT),
-
-                copy: true,
-
-                list: true,
-                list_with_recursive: true,
-                list_has_etag: true,
-                list_has_content_length: true,
-                list_has_content_md5: true,
-                list_has_content_type: true,
-                list_has_last_modified: true,
-
-                presign: self.has_sas_token,
-                presign_stat: self.has_sas_token,
-                presign_read: self.has_sas_token,
-                presign_write: self.has_sas_token,
-
-                shared: true,
-
-                ..Default::default()
-            });
-
-        am.into()
+        self.core.info.clone()
     }
 
     async fn stat(&self, path: &str, args: OpStat) -> Result<RpStat> {
@@ -563,10 +476,13 @@ impl Access for AzblobBackend {
             StatusCode::OK => {
                 let headers = resp.headers();
                 let mut meta = parse_into_metadata(path, headers)?;
+                if let Some(version_id) = parse_header_to_str(headers, X_MS_VERSION_ID)? {
+                    meta.set_version(version_id);
+                }
 
                 let user_meta = parse_prefixed_headers(headers, X_MS_META_PREFIX);
                 if !user_meta.is_empty() {
-                    meta.with_user_metadata(user_meta);
+                    meta = meta.with_user_metadata(user_meta);
                 }
 
                 Ok(RpStat::new(meta))
@@ -595,8 +511,8 @@ impl Access for AzblobBackend {
             AzblobWriters::Two(oio::AppendWriter::new(w))
         } else {
             AzblobWriters::One(oio::BlockWriter::new(
+                self.core.info.clone(),
                 w,
-                args.executor().cloned(),
                 args.concurrent(),
             ))
         };
@@ -634,18 +550,24 @@ impl Access for AzblobBackend {
     }
 
     async fn presign(&self, path: &str, args: OpPresign) -> Result<RpPresign> {
-        let mut req = match args.operation() {
-            PresignOperation::Stat(v) => self.core.azblob_head_blob_request(path, v)?,
+        let req = match args.operation() {
+            PresignOperation::Stat(v) => self.core.azblob_head_blob_request(path, v),
             PresignOperation::Read(v) => {
                 self.core
-                    .azblob_get_blob_request(path, BytesRange::default(), v)?
+                    .azblob_get_blob_request(path, BytesRange::default(), v)
             }
             PresignOperation::Write(_) => {
                 self.core
-                    .azblob_put_blob_request(path, None, &OpWrite::default(), Buffer::new())?
+                    .azblob_put_blob_request(path, None, &OpWrite::default(), Buffer::new())
             }
+            PresignOperation::Delete(_) => Err(Error::new(
+                ErrorKind::Unsupported,
+                "operation is not supported",
+            )),
         };
 
+        let mut req = req?;
+
         self.core.sign_query(&mut req).await?;
 
         let (parts, _) = req.into_parts();
@@ -657,101 +579,3 @@ impl Access for AzblobBackend {
         )))
     }
 }
-
-#[cfg(test)]
-mod tests {
-    use super::infer_storage_name_from_endpoint;
-    use super::AzblobBuilder;
-
-    #[test]
-    fn test_infer_storage_name_from_endpoint() {
-        let endpoint = "https://account.blob.core.windows.net";
-        let storage_name = infer_storage_name_from_endpoint(endpoint);
-        assert_eq!(storage_name, Some("account".to_string()));
-    }
-
-    #[test]
-    fn test_infer_storage_name_from_endpoint_with_trailing_slash() {
-        let endpoint = "https://account.blob.core.windows.net/";
-        let storage_name = infer_storage_name_from_endpoint(endpoint);
-        assert_eq!(storage_name, Some("account".to_string()));
-    }
-
-    #[test]
-    fn test_builder_from_connection_string() {
-        let builder = AzblobBuilder::from_connection_string(
-            r#"
-DefaultEndpointsProtocol=http;AccountName=devstoreaccount1;
-AccountKey=Eby8vdM02xNOcqFlqUwJPLlmEtlCDXJ1OUzFT50uSRZ6IFsuFq2UVErCz4I6tq/K1SZFPTOtr/KBHBeksoGMGw==;
-BlobEndpoint=http://127.0.0.1:10000/devstoreaccount1;
-QueueEndpoint=http://127.0.0.1:10001/devstoreaccount1;
-TableEndpoint=http://127.0.0.1:10002/devstoreaccount1;
-        "#,
-        )
-        .expect("from connection string must succeed");
-
-        assert_eq!(
-            builder.config.endpoint.unwrap(),
-            "http://127.0.0.1:10000/devstoreaccount1"
-        );
-        assert_eq!(builder.config.account_name.unwrap(), "devstoreaccount1");
-        assert_eq!(builder.config.account_key.unwrap(), "Eby8vdM02xNOcqFlqUwJPLlmEtlCDXJ1OUzFT50uSRZ6IFsuFq2UVErCz4I6tq/K1SZFPTOtr/KBHBeksoGMGw==");
-
-        let builder = AzblobBuilder::from_connection_string(
-            r#"
-DefaultEndpointsProtocol=https;
-AccountName=storagesample;
-AccountKey=account-key;
-EndpointSuffix=core.chinacloudapi.cn;
-        "#,
-        )
-        .expect("from connection string must succeed");
-
-        assert_eq!(
-            builder.config.endpoint.unwrap(),
-            "https://storagesample.blob.core.chinacloudapi.cn"
-        );
-        assert_eq!(builder.config.account_name.unwrap(), "storagesample");
-        assert_eq!(builder.config.account_key.unwrap(), "account-key")
-    }
-
-    #[test]
-    fn test_sas_from_connection_string() {
-        // Note, not a correct HMAC
-        let builder = AzblobBuilder::from_connection_string(
-            r#"
-BlobEndpoint=http://127.0.0.1:10000/devstoreaccount1;
-QueueEndpoint=http://127.0.0.1:10001/devstoreaccount1;
-TableEndpoint=http://127.0.0.1:10002/devstoreaccount1;
-SharedAccessSignature=sv=2021-01-01&ss=b&srt=c&sp=rwdlaciytfx&se=2022-01-01T11:00:14Z&st=2022-01-02T03:00:14Z&spr=https&sig=KEllk4N8f7rJfLjQCmikL2fRVt%2B%2Bl73UBkbgH%2FK3VGE%3D
-        "#,
-        )
-            .expect("from connection string must succeed");
-
-        assert_eq!(
-            builder.config.endpoint.unwrap(),
-            "http://127.0.0.1:10000/devstoreaccount1"
-        );
-        assert_eq!(builder.config.sas_token.unwrap(), "sv=2021-01-01&ss=b&srt=c&sp=rwdlaciytfx&se=2022-01-01T11:00:14Z&st=2022-01-02T03:00:14Z&spr=https&sig=KEllk4N8f7rJfLjQCmikL2fRVt%2B%2Bl73UBkbgH%2FK3VGE%3D");
-        assert_eq!(builder.config.account_name, None);
-        assert_eq!(builder.config.account_key, None);
-    }
-
-    #[test]
-    pub fn test_sas_preferred() {
-        let builder = AzblobBuilder::from_connection_string(
-            r#"
-BlobEndpoint=http://127.0.0.1:10000/devstoreaccount1;
-AccountName=storagesample;
-AccountKey=account-key;
-SharedAccessSignature=sv=2021-01-01&ss=b&srt=c&sp=rwdlaciytfx&se=2022-01-01T11:00:14Z&st=2022-01-02T03:00:14Z&spr=https&sig=KEllk4N8f7rJfLjQCmikL2fRVt%2B%2Bl73UBkbgH%2FK3VGE%3D
-        "#,
-        )
-            .expect("from connection string must succeed");
-
-        // SAS should be preferred over shared key
-        assert_eq!(builder.config.sas_token.unwrap(), "sv=2021-01-01&ss=b&srt=c&sp=rwdlaciytfx&se=2022-01-01T11:00:14Z&st=2022-01-02T03:00:14Z&spr=https&sig=KEllk4N8f7rJfLjQCmikL2fRVt%2B%2Bl73UBkbgH%2FK3VGE%3D");
-        assert_eq!(builder.config.account_name, None);
-        assert_eq!(builder.config.account_key, None);
-    }
-}
