@@ -15,11 +15,21 @@
 // specific language governing permissions and limitations
 // under the License.
 
-mod backend;
-pub use backend::AzblobBuilder as Azblob;
-pub use backend::AzblobConfig;
-
-mod core;
+#[cfg(feature = "services-azblob")]
+pub(crate) mod core;
+#[cfg(feature = "services-azblob")]
+mod delete;
+#[cfg(feature = "services-azblob")]
 mod error;
+#[cfg(feature = "services-azblob")]
 mod lister;
-mod writer;
+#[cfg(feature = "services-azblob")]
+pub(crate) mod writer;
+
+#[cfg(feature = "services-azblob")]
+mod backend;
+#[cfg(feature = "services-azblob")]
+pub use backend::AzblobBuilder as Azblob;
+
+mod config;
+pub use config::AzblobConfig;
