@@ -17,6 +17,9 @@
 
 use std::sync::Arc;
 
+use bytes::Buf;
+use http::HeaderMap;
+use http::HeaderValue;
 use http::StatusCode;
 
 use super::core::*;
@@ -41,6 +44,23 @@ impl CosWriter {
             op,
         }
     }
+
+    fn parse_metadata(headers: &HeaderMap<HeaderValue>) -> Result<Metadata> {
+        let mut meta = Metadata::default();
+        if let Some(etag) = parse_etag(headers)? {
+            meta.set_etag(etag);
+        }
+        if let Some(md5) = parse_content_md5(headers)? {
+            meta.set_content_md5(md5);
+        }
+        if let Some(version) = parse_header_to_str(headers, constants::X_COS_VERSION_ID)? {
+            if version != "null" {
+                meta.set_version(version);
+            }
+        }
+
+        Ok(meta)
+    }
 }
 
 impl oio::MultipartWrite for CosWriter {
@@ -53,10 +73,12 @@ impl oio::MultipartWrite for CosWriter {
 
         let resp = self.core.send(req).await?;
 
+        let meta = Self::parse_metadata(resp.headers())?;
+
         let status = resp.status();
 
         match status {
-            StatusCode::CREATED | StatusCode::OK => Ok(Metadata::default()),
+            StatusCode::CREATED | StatusCode::OK => Ok(meta),
             _ => Err(parse_error(resp)),
         }
     }
@@ -134,15 +156,22 @@ impl oio::MultipartWrite for CosWriter {
             })
             .collect();
 
-        let resp = self
+        let mut resp = self
             .core
             .cos_complete_multipart_upload(&self.path, upload_id, parts)
             .await?;
 
+        let mut meta = Self::parse_metadata(resp.headers())?;
+
+        let result: CompleteMultipartUploadResult =
+            quick_xml::de::from_reader(resp.body_mut().reader())
+                .map_err(new_xml_deserialize_error)?;
+        meta.set_etag(&result.etag);
+
         let status = resp.status();
 
         match status {
-            StatusCode::OK => Ok(Metadata::default()),
+            StatusCode::OK => Ok(meta),
             _ => Err(parse_error(resp)),
         }
     }
@@ -193,10 +222,12 @@ impl oio::AppendWrite for CosWriter {
 
         let resp = self.core.send(req).await?;
 
+        let meta = Self::parse_metadata(resp.headers())?;
+
         let status = resp.status();
 
         match status {
-            StatusCode::OK => Ok(Metadata::default()),
+            StatusCode::OK => Ok(meta),
             _ => Err(parse_error(resp)),
         }
     }
