@@ -19,6 +19,7 @@ use std::fmt::Debug;
 use std::fmt::Formatter;
 use std::sync::Arc;
 
+use hdfs_native::HdfsError;
 use hdfs_native::WriteOptions;
 use log::debug;
 
@@ -68,16 +69,16 @@ impl HdfsNativeBuilder {
         self
     }
 
-    /// Set url of this backend.
+    /// Set name_node of this backend.
     ///
     /// Valid format including:
     ///
     /// - `default`: using the default setting based on hadoop config.
     /// - `hdfs://127.0.0.1:9000`: connect to hdfs cluster.
-    pub fn url(mut self, url: &str) -> Self {
-        if !url.is_empty() {
+    pub fn name_node(mut self, name_node: &str) -> Self {
+        if !name_node.is_empty() {
             // Trim trailing `/` so that we can accept `http://127.0.0.1:9000/`
-            self.config.url = Some(url.trim_end_matches('/').to_string())
+            self.config.name_node = Some(name_node.trim_end_matches('/').to_string())
         }
 
         self
@@ -99,25 +100,24 @@ impl Builder for HdfsNativeBuilder {
     fn build(self) -> Result<impl Access> {
         debug!("backend build started: {:?}", &self);
 
-        let url = match &self.config.url {
+        let name_node = match &self.config.name_node {
             Some(v) => v,
             None => {
-                return Err(Error::new(ErrorKind::ConfigInvalid, "url is empty")
+                return Err(Error::new(ErrorKind::ConfigInvalid, "name_node is empty")
                     .with_context("service", Scheme::HdfsNative));
             }
         };
 
         let root = normalize_root(&self.config.root.unwrap_or_default());
-        debug!("backend use root {}", root);
+        debug!("backend use root {root}");
 
-        let client = hdfs_native::Client::new(url).map_err(parse_hdfs_error)?;
+        let client = hdfs_native::Client::new(name_node).map_err(parse_hdfs_error)?;
 
         // need to check if root dir exists, create if not
-
         Ok(HdfsNativeBackend {
             root,
             client: Arc::new(client),
-            _enable_append: self.config.enable_append,
+            enable_append: self.config.enable_append,
         })
     }
 }
@@ -126,7 +126,7 @@ impl Builder for HdfsNativeBuilder {
 // fn tmp_file_of(path: &str) -> String {
 //     let name = get_basename(path);
 //     let uuid = Uuid::new_v4().to_string();
-//
+
 //     format!("{name}.{uuid}")
 // }
 
@@ -135,7 +135,7 @@ impl Builder for HdfsNativeBuilder {
 pub struct HdfsNativeBackend {
     pub root: String,
     pub client: Arc<hdfs_native::Client>,
-    _enable_append: bool,
+    enable_append: bool,
 }
 
 /// hdfs_native::Client is thread-safe.
@@ -147,21 +147,24 @@ impl Access for HdfsNativeBackend {
     type Writer = HdfsNativeWriter;
     type Lister = Option<HdfsNativeLister>;
     type Deleter = oio::OneShotDeleter<HdfsNativeDeleter>;
-    type BlockingReader = ();
-    type BlockingWriter = ();
-    type BlockingLister = ();
-    type BlockingDeleter = ();
 
     fn info(&self) -> Arc<AccessorInfo> {
-        let mut am = AccessorInfo::default();
+        let am = AccessorInfo::default();
         am.set_scheme(Scheme::HdfsNative)
             .set_root(&self.root)
             .set_native_capability(Capability {
                 stat: true,
-                stat_has_last_modified: true,
-                stat_has_content_length: true,
 
+                read: true,
+
+                write: true,
+                write_can_append: self.enable_append,
+
+                create_dir: true,
                 delete: true,
+
+                list: true,
+
                 rename: true,
 
                 shared: true,
@@ -179,6 +182,7 @@ impl Access for HdfsNativeBackend {
             .mkdirs(&p, 0o777, true)
             .await
             .map_err(parse_hdfs_error)?;
+
         Ok(RpCreateDir::default())
     }
 
@@ -207,28 +211,58 @@ impl Access for HdfsNativeBackend {
         Ok(RpStat::new(metadata))
     }
 
-    async fn read(&self, path: &str, _args: OpRead) -> Result<(RpRead, Self::Reader)> {
+    async fn read(&self, path: &str, args: OpRead) -> Result<(RpRead, Self::Reader)> {
         let p = build_rooted_abs_path(&self.root, path);
 
         let f = self.client.read(&p).await.map_err(parse_hdfs_error)?;
 
-        let r = HdfsNativeReader::new(f);
+        let r = HdfsNativeReader::new(
+            f,
+            args.range().offset() as _,
+            args.range().size().unwrap_or(u64::MAX) as _,
+        );
 
         Ok((RpRead::new(), r))
     }
 
-    async fn write(&self, path: &str, _args: OpWrite) -> Result<(RpWrite, Self::Writer)> {
-        let p = build_rooted_abs_path(&self.root, path);
+    async fn write(&self, path: &str, args: OpWrite) -> Result<(RpWrite, Self::Writer)> {
+        let target_path = build_rooted_abs_path(&self.root, path);
+        let mut initial_size = 0;
 
-        let f = self
-            .client
-            .create(&p, WriteOptions::default())
-            .await
-            .map_err(parse_hdfs_error)?;
+        let target_exists = match self.client.get_file_info(&target_path).await {
+            Ok(status) => {
+                initial_size = status.length as u64;
+                true
+            }
+            Err(err) => match &err {
+                HdfsError::FileNotFound(_) => false,
+                _ => return Err(parse_hdfs_error(err)),
+            },
+        };
 
-        let w = HdfsNativeWriter::new(f);
+        let f = if target_exists {
+            if args.append() {
+                assert!(self.enable_append, "append is not enabled");
+                self.client
+                    .append(&target_path)
+                    .await
+                    .map_err(parse_hdfs_error)?
+            } else {
+                initial_size = 0;
+                self.client
+                    .create(&target_path, WriteOptions::default().overwrite(true))
+                    .await
+                    .map_err(parse_hdfs_error)?
+            }
+        } else {
+            initial_size = 0;
+            self.client
+                .create(&target_path, WriteOptions::default())
+                .await
+                .map_err(parse_hdfs_error)?
+        };
 
-        Ok((RpWrite::new(), w))
+        Ok((RpWrite::new(), HdfsNativeWriter::new(f, initial_size)))
     }
 
     async fn delete(&self) -> Result<(RpDelete, Self::Deleter)> {
@@ -239,17 +273,66 @@ impl Access for HdfsNativeBackend {
     }
 
     async fn list(&self, path: &str, _args: OpList) -> Result<(RpList, Self::Lister)> {
-        let p = build_rooted_abs_path(&self.root, path);
-        let l = HdfsNativeLister::new(p, self.client.clone());
-        Ok((RpList::default(), Some(l)))
+        let p: String = build_rooted_abs_path(&self.root, path);
+
+        let isdir = match self.client.get_file_info(&p).await {
+            Ok(status) => status.isdir,
+            Err(err) => {
+                return match &err {
+                    HdfsError::FileNotFound(_) => Ok((RpList::default(), None)),
+                    _ => Err(parse_hdfs_error(err)),
+                };
+            }
+        };
+        let current_path = if isdir {
+            if !path.ends_with("/") {
+                Some(path.to_string() + "/")
+            } else {
+                Some(path.to_string())
+            }
+        } else {
+            None
+        };
+
+        Ok((
+            RpList::default(),
+            Some(HdfsNativeLister::new(
+                &self.root,
+                &self.client,
+                &p,
+                current_path,
+            )),
+        ))
     }
 
     async fn rename(&self, from: &str, to: &str, _args: OpRename) -> Result<RpRename> {
         let from_path = build_rooted_abs_path(&self.root, from);
         let to_path = build_rooted_abs_path(&self.root, to);
+        match self.client.get_file_info(&to_path).await {
+            Ok(status) => {
+                if status.isdir {
+                    return Err(Error::new(ErrorKind::IsADirectory, "path should be a file")
+                        .with_context("input", &to_path));
+                } else {
+                    self.client
+                        .delete(&to_path, true)
+                        .await
+                        .map_err(parse_hdfs_error)?;
+                }
+            }
+            Err(err) => match &err {
+                HdfsError::FileNotFound(_) => {
+                    self.client
+                        .create(&to_path, WriteOptions::default().create_parent(true))
+                        .await
+                        .map_err(parse_hdfs_error)?;
+                }
+                _ => return Err(parse_hdfs_error(err)),
+            },
+        };
 
         self.client
-            .rename(&from_path, &to_path, false)
+            .rename(&from_path, &to_path, true)
             .await
             .map_err(parse_hdfs_error)?;
 
