@@ -16,6 +16,7 @@
 // under the License.
 
 use std::fmt::Debug;
+use std::sync::Arc;
 
 use bytes::Bytes;
 use http::header;
@@ -28,13 +29,13 @@ use crate::raw::*;
 use crate::*;
 
 pub struct HuggingfaceCore {
+    pub info: Arc<AccessorInfo>,
+
     pub repo_type: RepoType,
     pub repo_id: String,
     pub revision: String,
     pub root: String,
     pub token: Option<String>,
-
-    pub client: HttpClient,
 }
 
 impl Debug for HuggingfaceCore {
@@ -66,7 +67,8 @@ impl HuggingfaceCore {
         };
 
         let mut req = Request::post(&url);
-
+        // Inject operation to the request.
+        req = req.extension(Operation::Stat);
         if let Some(token) = &self.token {
             let auth_header_content = format_authorization_by_bearer(token)?;
             req = req.header(header::AUTHORIZATION, auth_header_content);
@@ -80,7 +82,7 @@ impl HuggingfaceCore {
             .body(Buffer::from(Bytes::from(req_body)))
             .map_err(new_request_build_error)?;
 
-        self.client.send(req).await
+        self.info.http_client().send(req).await
     }
 
     pub async fn hf_list(&self, path: &str, recursive: bool) -> Result<Response<Buffer>> {
@@ -108,7 +110,8 @@ impl HuggingfaceCore {
         }
 
         let mut req = Request::get(&url);
-
+        // Inject operation to the request.
+        req = req.extension(Operation::List);
         if let Some(token) = &self.token {
             let auth_header_content = format_authorization_by_bearer(token)?;
             req = req.header(header::AUTHORIZATION, auth_header_content);
@@ -116,7 +119,7 @@ impl HuggingfaceCore {
 
         let req = req.body(Buffer::new()).map_err(new_request_build_error)?;
 
-        self.client.send(req).await
+        self.info.http_client().send(req).await
     }
 
     pub async fn hf_resolve(
@@ -154,10 +157,11 @@ impl HuggingfaceCore {
         if !range.is_full() {
             req = req.header(header::RANGE, range.to_header());
         }
-
+        // Inject operation to the request.
+        let req = req.extension(Operation::Read);
         let req = req.body(Buffer::new()).map_err(new_request_build_error)?;
 
-        self.client.fetch(req).await
+        self.info.http_client().fetch(req).await
     }
 }
 
