@@ -15,5 +15,19 @@
 // specific language governing permissions and limitations
 // under the License.
 
+#[cfg(feature = "services-mini-moka")]
 mod backend;
+#[cfg(feature = "services-mini-moka")]
+mod core;
+#[cfg(feature = "services-mini-moka")]
+mod delete;
+#[cfg(feature = "services-mini-moka")]
+mod lister;
+#[cfg(feature = "services-mini-moka")]
+mod writer;
+
+#[cfg(feature = "services-mini-moka")]
 pub use backend::MiniMokaBuilder as MiniMoka;
+
+mod config;
+pub use config::MiniMokaConfig;
