@@ -17,8 +17,22 @@
 
 #[cfg(feature = "services-moka")]
 mod backend;
+#[cfg(feature = "services-moka")]
+mod core;
+#[cfg(feature = "services-moka")]
+mod delete;
+#[cfg(feature = "services-moka")]
+mod lister;
+#[cfg(feature = "services-moka")]
+mod writer;
+
 #[cfg(feature = "services-moka")]
 pub use backend::MokaBuilder as Moka;
+#[cfg(feature = "services-moka")]
+pub use backend::MokaCacheBuilder;
+
+#[cfg(feature = "services-moka")]
+pub use core::MokaValue;
 
 mod config;
 pub use config::MokaConfig;
