@@ -37,6 +37,8 @@ use crate::*;
 
 impl Configurator for PcloudConfig {
     type Builder = PcloudBuilder;
+
+    #[allow(deprecated)]
     fn into_builder(self) -> Self::Builder {
         PcloudBuilder {
             config: self,
@@ -51,6 +53,7 @@ impl Configurator for PcloudConfig {
 pub struct PcloudBuilder {
     config: PcloudConfig,
 
+    #[deprecated(since = "0.53.0", note = "Use `Operator::update_http_client` instead")]
     http_client: Option<HttpClient>,
 }
 
@@ -120,6 +123,8 @@ impl PcloudBuilder {
     ///
     /// This API is part of OpenDAL's Raw API. `HttpClient` could be changed
     /// during minor updates.
+    #[deprecated(since = "0.53.0", note = "Use `Operator::update_http_client` instead")]
+    #[allow(deprecated)]
     pub fn http_client(mut self, client: HttpClient) -> Self {
         self.http_client = Some(client);
         self
@@ -160,22 +165,44 @@ impl Builder for PcloudBuilder {
                 .with_context("service", Scheme::Pcloud)),
         }?;
 
-        let client = if let Some(client) = self.http_client {
-            client
-        } else {
-            HttpClient::new().map_err(|err| {
-                err.with_operation("Builder::build")
-                    .with_context("service", Scheme::Pcloud)
-            })?
-        };
-
         Ok(PcloudBackend {
             core: Arc::new(PcloudCore {
+                info: {
+                    let am = AccessorInfo::default();
+                    am.set_scheme(Scheme::Pcloud)
+                        .set_root(&root)
+                        .set_native_capability(Capability {
+                            stat: true,
+
+                            create_dir: true,
+
+                            read: true,
+
+                            write: true,
+
+                            delete: true,
+                            rename: true,
+                            copy: true,
+
+                            list: true,
+
+                            shared: true,
+
+                            ..Default::default()
+                        });
+
+                    // allow deprecated api here for compatibility
+                    #[allow(deprecated)]
+                    if let Some(client) = self.http_client {
+                        am.update_http_client(|_| client);
+                    }
+
+                    am.into()
+                },
                 root,
                 endpoint: self.config.endpoint.clone(),
                 username,
                 password,
-                client,
             }),
         })
     }
@@ -192,40 +219,9 @@ impl Access for PcloudBackend {
     type Writer = PcloudWriters;
     type Lister = oio::PageLister<PcloudLister>;
     type Deleter = oio::OneShotDeleter<PcloudDeleter>;
-    type BlockingReader = ();
-    type BlockingWriter = ();
-    type BlockingLister = ();
-    type BlockingDeleter = ();
 
     fn info(&self) -> Arc<AccessorInfo> {
-        let mut am = AccessorInfo::default();
-        am.set_scheme(Scheme::Pcloud)
-            .set_root(&self.core.root)
-            .set_native_capability(Capability {
-                stat: true,
-                stat_has_content_length: true,
-                stat_has_last_modified: true,
-
-                create_dir: true,
-
-                read: true,
-
-                write: true,
-
-                delete: true,
-                rename: true,
-                copy: true,
-
-                list: true,
-                list_has_content_length: true,
-                list_has_last_modified: true,
-
-                shared: true,
-
-                ..Default::default()
-            });
-
-        am.into()
+        self.core.info.clone()
     }
 
     async fn create_dir(&self, path: &str, _: OpCreateDir) -> Result<RpCreateDir> {
