@@ -19,8 +19,6 @@ use std::fmt::Debug;
 use std::fmt::Formatter;
 use std::sync::Arc;
 
-use tokio::task;
-
 use crate::raw::adapters::kv;
 use crate::raw::*;
 use crate::services::RedbConfig;
@@ -33,7 +31,10 @@ use crate::*;
 impl Configurator for RedbConfig {
     type Builder = RedbBuilder;
     fn into_builder(self) -> Self::Builder {
-        RedbBuilder { config: self }
+        RedbBuilder {
+            config: self,
+            database: None,
+        }
     }
 }
 
@@ -42,16 +43,47 @@ impl Configurator for RedbConfig {
 #[derive(Default, Debug)]
 pub struct RedbBuilder {
     config: RedbConfig,
+
+    database: Option<Arc<redb::Database>>,
 }
 
 impl RedbBuilder {
+    /// Set the database for Redb.
+    ///
+    /// This method should be called when you want to
+    /// use multiple tables of one database because
+    /// Redb doesn't allow opening a database that have been opened.
+    ///
+    /// <div class="warning">
+    ///
+    /// `datadir` and `database` should not be set simultaneously.
+    /// If both are set, `database` will take precedence.
+    ///
+    /// </div>
+    pub fn database(mut self, db: Arc<redb::Database>) -> Self {
+        self.database = Some(db);
+        self
+    }
+
     /// Set the path to the redb data directory. Will create if not exists.
+    ///
+    ///
+    /// <div class="warning">
+    ///
+    /// Opening redb database via `datadir` takes away the ability to access multiple redb tables.
+    /// If you need to access multiple redb tables, the correct solution is to
+    /// create an `Arc<redb::database>` beforehand and then share it via [`database`]
+    /// with multiple builders where every builder will open one redb table.
+    ///
+    /// </div>
+    ///
+    /// [`database`]: RedbBuilder::database
     pub fn datadir(mut self, path: &str) -> Self {
         self.config.datadir = Some(path.into());
         self
     }
 
-    /// Set the table name for Redb.
+    /// Set the table name for Redb. Will create if not exists.
     pub fn table(mut self, table: &str) -> Self {
         self.config.table = Some(table.into());
         self
@@ -69,22 +101,30 @@ impl Builder for RedbBuilder {
     type Config = RedbConfig;
 
     fn build(self) -> Result<impl Access> {
-        let datadir_path = self.config.datadir.ok_or_else(|| {
-            Error::new(ErrorKind::ConfigInvalid, "datadir is required but not set")
-                .with_context("service", Scheme::Redb)
-        })?;
-
         let table_name = self.config.table.ok_or_else(|| {
             Error::new(ErrorKind::ConfigInvalid, "table is required but not set")
                 .with_context("service", Scheme::Redb)
         })?;
 
-        let db = redb::Database::create(&datadir_path).map_err(parse_database_error)?;
+        let (datadir, db) = if let Some(db) = self.database {
+            (None, db)
+        } else {
+            let datadir = self.config.datadir.ok_or_else(|| {
+                Error::new(ErrorKind::ConfigInvalid, "datadir is required but not set")
+                    .with_context("service", Scheme::Redb)
+            })?;
 
-        let db = Arc::new(db);
+            let db = redb::Database::create(&datadir)
+                .map_err(parse_database_error)?
+                .into();
+
+            (Some(datadir), db)
+        };
+
+        create_table(&db, &table_name)?;
 
         Ok(RedbBackend::new(Adapter {
-            datadir: datadir_path,
+            datadir,
             table: table_name,
             db,
         })
@@ -97,7 +137,7 @@ pub type RedbBackend = kv::Backend<Adapter>;
 
 #[derive(Clone)]
 pub struct Adapter {
-    datadir: String,
+    datadir: Option<String>,
     table: String,
     db: Arc<redb::Database>,
 }
@@ -116,11 +156,10 @@ impl kv::Adapter for Adapter {
     fn info(&self) -> kv::Info {
         kv::Info::new(
             Scheme::Redb,
-            &self.datadir,
+            &self.table,
             Capability {
                 read: true,
                 write: true,
-                blocking: true,
                 shared: false,
                 ..Default::default()
             },
@@ -128,16 +167,6 @@ impl kv::Adapter for Adapter {
     }
 
     async fn get(&self, path: &str) -> Result<Option<Buffer>> {
-        let cloned_self = self.clone();
-        let cloned_path = path.to_string();
-
-        task::spawn_blocking(move || cloned_self.blocking_get(cloned_path.as_str()))
-            .await
-            .map_err(new_task_join_error)
-            .and_then(|inner_result| inner_result)
-    }
-
-    fn blocking_get(&self, path: &str) -> Result<Option<Buffer>> {
         let read_txn = self.db.begin_read().map_err(parse_transaction_error)?;
 
         let table_define: redb::TableDefinition<&str, &[u8]> =
@@ -156,16 +185,6 @@ impl kv::Adapter for Adapter {
     }
 
     async fn set(&self, path: &str, value: Buffer) -> Result<()> {
-        let cloned_self = self.clone();
-        let cloned_path = path.to_string();
-
-        task::spawn_blocking(move || cloned_self.blocking_set(cloned_path.as_str(), value))
-            .await
-            .map_err(new_task_join_error)
-            .and_then(|inner_result| inner_result)
-    }
-
-    fn blocking_set(&self, path: &str, value: Buffer) -> Result<()> {
         let write_txn = self.db.begin_write().map_err(parse_transaction_error)?;
 
         let table_define: redb::TableDefinition<&str, &[u8]> =
@@ -186,16 +205,6 @@ impl kv::Adapter for Adapter {
     }
 
     async fn delete(&self, path: &str) -> Result<()> {
-        let cloned_self = self.clone();
-        let cloned_path = path.to_string();
-
-        task::spawn_blocking(move || cloned_self.blocking_delete(cloned_path.as_str()))
-            .await
-            .map_err(new_task_join_error)
-            .and_then(|inner_result| inner_result)
-    }
-
-    fn blocking_delete(&self, path: &str) -> Result<()> {
         let write_txn = self.db.begin_write().map_err(parse_transaction_error)?;
 
         let table_define: redb::TableDefinition<&str, &[u8]> =
@@ -238,3 +247,35 @@ fn parse_database_error(e: redb::DatabaseError) -> Error {
 fn parse_commit_error(e: redb::CommitError) -> Error {
     Error::new(ErrorKind::Unexpected, "error from redb").set_source(e)
 }
+
+/// Check if a table exists, otherwise create it.
+fn create_table(db: &redb::Database, table: &str) -> Result<()> {
+    // Only one `WriteTransaction` is permitted at same time,
+    // applying new one will block until it available.
+    //
+    // So we first try checking table existence via `ReadTransaction`.
+    {
+        let read_txn = db.begin_read().map_err(parse_transaction_error)?;
+
+        let table_define: redb::TableDefinition<&str, &[u8]> = redb::TableDefinition::new(table);
+
+        match read_txn.open_table(table_define) {
+            Ok(_) => return Ok(()),
+            Err(redb::TableError::TableDoesNotExist(_)) => (),
+            Err(e) => return Err(parse_table_error(e)),
+        }
+    }
+
+    {
+        let write_txn = db.begin_write().map_err(parse_transaction_error)?;
+
+        let table_define: redb::TableDefinition<&str, &[u8]> = redb::TableDefinition::new(table);
+
+        write_txn
+            .open_table(table_define)
+            .map_err(parse_table_error)?;
+        write_txn.commit().map_err(parse_commit_error)?;
+    }
+
+    Ok(())
+}
