@@ -121,7 +121,7 @@ impl Builder for SwiftBuilder {
         debug!("backend build started: {:?}", &self);
 
         let root = normalize_root(&self.config.root.unwrap_or_default());
-        debug!("backend use root {}", root);
+        debug!("backend use root {root}");
 
         let endpoint = match self.config.endpoint {
             Some(endpoint) => {
@@ -152,15 +152,35 @@ impl Builder for SwiftBuilder {
 
         let token = self.config.token.unwrap_or_default();
 
-        let client = HttpClient::new()?;
-
         Ok(SwiftBackend {
             core: Arc::new(SwiftCore {
+                info: {
+                    let am = AccessorInfo::default();
+                    am.set_scheme(Scheme::Swift)
+                        .set_root(&root)
+                        .set_native_capability(Capability {
+                            stat: true,
+                            read: true,
+
+                            write: true,
+                            write_can_empty: true,
+                            write_with_user_metadata: true,
+
+                            delete: true,
+
+                            list: true,
+                            list_with_recursive: true,
+
+                            shared: true,
+
+                            ..Default::default()
+                        });
+                    am.into()
+                },
                 root,
                 endpoint,
                 container,
                 token,
-                client,
             }),
         })
     }
@@ -177,47 +197,9 @@ impl Access for SwiftBackend {
     type Writer = oio::OneShotWriter<SwiftWriter>;
     type Lister = oio::PageLister<SwiftLister>;
     type Deleter = oio::OneShotDeleter<SwfitDeleter>;
-    type BlockingReader = ();
-    type BlockingWriter = ();
-    type BlockingLister = ();
-    type BlockingDeleter = ();
 
     fn info(&self) -> Arc<AccessorInfo> {
-        let mut am = AccessorInfo::default();
-        am.set_scheme(Scheme::Swift)
-            .set_root(&self.core.root)
-            .set_native_capability(Capability {
-                stat: true,
-                stat_has_cache_control: true,
-                stat_has_content_length: true,
-                stat_has_content_type: true,
-                stat_has_content_encoding: true,
-                stat_has_content_range: true,
-                stat_has_etag: true,
-                stat_has_content_md5: true,
-                stat_has_last_modified: true,
-                stat_has_content_disposition: true,
-                stat_has_user_metadata: true,
-                read: true,
-
-                write: true,
-                write_can_empty: true,
-                write_with_user_metadata: true,
-
-                delete: true,
-
-                list: true,
-                list_with_recursive: true,
-                list_has_content_length: true,
-                list_has_content_md5: true,
-                list_has_content_type: true,
-                list_has_last_modified: true,
-
-                shared: true,
-
-                ..Default::default()
-            });
-        am.into()
+        self.core.info.clone()
     }
 
     async fn stat(&self, path: &str, _args: OpStat) -> Result<RpStat> {
@@ -229,7 +211,7 @@ impl Access for SwiftBackend {
                 let mut meta = parse_into_metadata(path, headers)?;
                 let user_meta = parse_prefixed_headers(headers, "x-object-meta-");
                 if !user_meta.is_empty() {
-                    meta.with_user_metadata(user_meta);
+                    meta = meta.with_user_metadata(user_meta);
                 }
 
                 Ok(RpStat::new(meta))
