@@ -116,7 +116,7 @@ impl Builder for TikvBuilder {
             return Err(
                 Error::new(ErrorKind::ConfigInvalid, "invalid tls configuration")
                     .with_context("service", Scheme::Tikv)
-                    .with_context("endpoints", format!("{:?}", endpoints)),
+                    .with_context("endpoints", format!("{endpoints:?}")),
             )?;
         }
 
@@ -194,7 +194,6 @@ impl kv::Adapter for Adapter {
             Capability {
                 read: true,
                 write: true,
-                blocking: false,
                 shared: true,
                 ..Default::default()
             },
