@@ -27,6 +27,7 @@ use log::debug;
 use super::core::parse_blob;
 use super::core::Blob;
 use super::core::VercelBlobCore;
+use super::delete::VercelBlobDeleter;
 use super::error::parse_error;
 use super::lister::VercelBlobLister;
 use super::writer::VercelBlobWriter;
@@ -37,6 +38,8 @@ use crate::*;
 
 impl Configurator for VercelBlobConfig {
     type Builder = VercelBlobBuilder;
+
+    #[allow(deprecated)]
     fn into_builder(self) -> Self::Builder {
         VercelBlobBuilder {
             config: self,
@@ -51,6 +54,7 @@ impl Configurator for VercelBlobConfig {
 pub struct VercelBlobBuilder {
     config: VercelBlobConfig,
 
+    #[deprecated(since = "0.53.0", note = "Use `Operator::update_http_client` instead")]
     http_client: Option<HttpClient>,
 }
 
@@ -94,6 +98,8 @@ impl VercelBlobBuilder {
     ///
     /// This API is part of OpenDAL's Raw API. `HttpClient` could be changed
     /// during minor updates.
+    #[deprecated(since = "0.53.0", note = "Use `Operator::update_http_client` instead")]
+    #[allow(deprecated)]
     pub fn http_client(mut self, client: HttpClient) -> Self {
         self.http_client = Some(client);
         self
@@ -118,20 +124,44 @@ impl Builder for VercelBlobBuilder {
                 .with_context("service", Scheme::VercelBlob));
         };
 
-        let client = if let Some(client) = self.http_client {
-            client
-        } else {
-            HttpClient::new().map_err(|err| {
-                err.with_operation("Builder::build")
-                    .with_context("service", Scheme::VercelBlob)
-            })?
-        };
-
         Ok(VercelBlobBackend {
             core: Arc::new(VercelBlobCore {
+                info: {
+                    let am = AccessorInfo::default();
+                    am.set_scheme(Scheme::VercelBlob)
+                        .set_root(&root)
+                        .set_native_capability(Capability {
+                            stat: true,
+
+                            read: true,
+
+                            write: true,
+                            write_can_empty: true,
+                            write_can_multi: true,
+                            write_multi_min_size: Some(5 * 1024 * 1024),
+
+                            copy: true,
+
+                            list: true,
+                            list_with_limit: true,
+
+                            delete: true,
+
+                            shared: true,
+
+                            ..Default::default()
+                        });
+
+                    // allow deprecated api here for compatibility
+                    #[allow(deprecated)]
+                    if let Some(client) = self.http_client {
+                        am.update_http_client(|_| client);
+                    }
+
+                    am.into()
+                },
                 root,
                 token,
-                client,
             }),
         })
     }
@@ -147,45 +177,10 @@ impl Access for VercelBlobBackend {
     type Reader = HttpBody;
     type Writer = VercelBlobWriters;
     type Lister = oio::PageLister<VercelBlobLister>;
-    type Deleter = ();
-    type BlockingReader = ();
-    type BlockingWriter = ();
-    type BlockingLister = ();
-    type BlockingDeleter = ();
+    type Deleter = oio::OneShotDeleter<VercelBlobDeleter>;
 
     fn info(&self) -> Arc<AccessorInfo> {
-        let mut am = AccessorInfo::default();
-        am.set_scheme(Scheme::VercelBlob)
-            .set_root(&self.core.root)
-            .set_native_capability(Capability {
-                stat: true,
-                stat_has_content_type: true,
-                stat_has_content_length: true,
-                stat_has_last_modified: true,
-                stat_has_content_disposition: true,
-
-                read: true,
-
-                write: true,
-                write_can_empty: true,
-                write_can_multi: true,
-                write_multi_min_size: Some(5 * 1024 * 1024),
-
-                copy: true,
-
-                list: true,
-                list_with_limit: true,
-                list_has_content_type: true,
-                list_has_content_length: true,
-                list_has_last_modified: true,
-                list_has_content_disposition: true,
-
-                shared: true,
-
-                ..Default::default()
-            });
-
-        am.into()
+        self.core.info.clone()
     }
 
     async fn stat(&self, path: &str, _args: OpStat) -> Result<RpStat> {
@@ -225,10 +220,9 @@ impl Access for VercelBlobBackend {
 
     async fn write(&self, path: &str, args: OpWrite) -> Result<(RpWrite, Self::Writer)> {
         let concurrent = args.concurrent();
-        let executor = args.executor().cloned();
         let writer = VercelBlobWriter::new(self.core.clone(), args, path.to_string());
 
-        let w = oio::MultipartWriter::new(writer, executor, concurrent);
+        let w = oio::MultipartWriter::new(self.core.info.clone(), writer, concurrent);
 
         Ok((RpWrite::default(), w))
     }
@@ -248,4 +242,11 @@ impl Access for VercelBlobBackend {
         let l = VercelBlobLister::new(self.core.clone(), path, args.limit());
         Ok((RpList::default(), oio::PageLister::new(l)))
     }
+
+    async fn delete(&self) -> Result<(RpDelete, Self::Deleter)> {
+        Ok((
+            RpDelete::default(),
+            oio::OneShotDeleter::new(VercelBlobDeleter::new(self.core.clone())),
+        ))
+    }
 }
