@@ -82,28 +82,6 @@ pub struct Capability {
     pub stat_with_override_content_type: bool,
     /// Indicates if versions stat operations are supported.
     pub stat_with_version: bool,
-    /// Indicates whether cache control information is available in stat response
-    pub stat_has_cache_control: bool,
-    /// Indicates whether content disposition information is available in stat response
-    pub stat_has_content_disposition: bool,
-    /// Indicates whether content length information is available in stat response
-    pub stat_has_content_length: bool,
-    /// Indicates whether content MD5 checksum is available in stat response
-    pub stat_has_content_md5: bool,
-    /// Indicates whether content range information is available in stat response
-    pub stat_has_content_range: bool,
-    /// Indicates whether content type information is available in stat response
-    pub stat_has_content_type: bool,
-    /// Indicates whether content encoding information is available in stat response
-    pub stat_has_content_encoding: bool,
-    /// Indicates whether entity tag is available in stat response
-    pub stat_has_etag: bool,
-    /// Indicates whether last modified timestamp is available in stat response
-    pub stat_has_last_modified: bool,
-    /// Indicates whether version information is available in stat response
-    pub stat_has_version: bool,
-    /// Indicates whether user-defined metadata is available in stat response
-    pub stat_has_user_metadata: bool,
 
     /// Indicates if the operator supports read operations.
     pub read: bool,
@@ -157,15 +135,6 @@ pub struct Capability {
     /// Maximum total size supported for write operations.
     /// For example, Cloudflare D1 has a 1MB total size limit.
     pub write_total_max_size: Option<usize>,
-    /// Indicates whether content length information is available in write response.
-    /// by default, it should be `true` for all write operations.
-    pub write_has_content_length: bool,
-    /// Indicates whether last modified timestamp is available in write response
-    pub write_has_last_modified: bool,
-    /// Indicates whether entity tag is available in write response
-    pub write_has_etag: bool,
-    /// Indicates whether version information is available in write response
-    pub write_has_version: bool,
 
     /// Indicates if directory creation is supported.
     pub create_dir: bool,
@@ -198,26 +167,6 @@ pub struct Capability {
     pub list_with_versions: bool,
     /// Indicates if listing with deleted files included is supported.
     pub list_with_deleted: bool,
-    /// Indicates whether cache control information is available in list response
-    pub list_has_cache_control: bool,
-    /// Indicates whether content disposition information is available in list response
-    pub list_has_content_disposition: bool,
-    /// Indicates whether content length information is available in list response
-    pub list_has_content_length: bool,
-    /// Indicates whether content MD5 checksum is available in list response
-    pub list_has_content_md5: bool,
-    /// Indicates whether content range information is available in list response
-    pub list_has_content_range: bool,
-    /// Indicates whether content type information is available in list response
-    pub list_has_content_type: bool,
-    /// Indicates whether entity tag is available in list response
-    pub list_has_etag: bool,
-    /// Indicates whether last modified timestamp is available in list response
-    pub list_has_last_modified: bool,
-    /// Indicates whether version information is available in list response
-    pub list_has_version: bool,
-    /// Indicates whether user-defined metadata is available in list response
-    pub list_has_user_metadata: bool,
 
     /// Indicates if presigned URL generation is supported.
     pub presign: bool,
@@ -227,12 +176,11 @@ pub struct Capability {
     pub presign_stat: bool,
     /// Indicates if presigned URLs for write operations are supported.
     pub presign_write: bool,
+    /// Indicates if presigned URLs for delete operations are supported.
+    pub presign_delete: bool,
 
     /// Indicate if the operator supports shared access.
     pub shared: bool,
-
-    /// Indicates if blocking operations are supported.
-    pub blocking: bool,
 }
 
 impl Debug for Capability {
@@ -253,9 +201,7 @@ impl Debug for Capability {
         if self.shared {
             f.write_str("| Shared")?;
         }
-        if self.blocking {
-            f.write_str("| Blocking")?;
-        }
+
         Ok(())
     }
 }
