@@ -168,8 +168,6 @@ impl Operator {
         let op = match scheme {
             #[cfg(feature = "services-aliyun-drive")]
             Scheme::AliyunDrive => Self::from_iter::<services::AliyunDrive>(iter)?.finish(),
-            #[cfg(feature = "services-atomicserver")]
-            Scheme::Atomicserver => Self::from_iter::<services::Atomicserver>(iter)?.finish(),
             #[cfg(feature = "services-alluxio")]
             Scheme::Alluxio => Self::from_iter::<services::Alluxio>(iter)?.finish(),
             #[cfg(feature = "services-compfs")]
@@ -182,8 +180,6 @@ impl Operator {
             Scheme::YandexDisk => Self::from_iter::<services::YandexDisk>(iter)?.finish(),
             #[cfg(feature = "services-pcloud")]
             Scheme::Pcloud => Self::from_iter::<services::Pcloud>(iter)?.finish(),
-            #[cfg(feature = "services-chainsafe")]
-            Scheme::Chainsafe => Self::from_iter::<services::Chainsafe>(iter)?.finish(),
             #[cfg(feature = "services-azblob")]
             Scheme::Azblob => Self::from_iter::<services::Azblob>(iter)?.finish(),
             #[cfg(feature = "services-azdls")]
@@ -228,8 +224,6 @@ impl Operator {
             Scheme::Ipfs => Self::from_iter::<services::Ipfs>(iter)?.finish(),
             #[cfg(feature = "services-ipmfs")]
             Scheme::Ipmfs => Self::from_iter::<services::Ipmfs>(iter)?.finish(),
-            #[cfg(feature = "services-icloud")]
-            Scheme::Icloud => Self::from_iter::<services::Icloud>(iter)?.finish(),
             #[cfg(feature = "services-memcached")]
             Scheme::Memcached => Self::from_iter::<services::Memcached>(iter)?.finish(),
             #[cfg(feature = "services-memory")]
@@ -268,8 +262,6 @@ impl Operator {
             Scheme::Sled => Self::from_iter::<services::Sled>(iter)?.finish(),
             #[cfg(feature = "services-sqlite")]
             Scheme::Sqlite => Self::from_iter::<services::Sqlite>(iter)?.finish(),
-            #[cfg(feature = "services-supabase")]
-            Scheme::Supabase => Self::from_iter::<services::Supabase>(iter)?.finish(),
             #[cfg(feature = "services-swift")]
             Scheme::Swift => Self::from_iter::<services::Swift>(iter)?.finish(),
             #[cfg(feature = "services-tikv")]
@@ -290,8 +282,6 @@ impl Operator {
             Scheme::HdfsNative => Self::from_iter::<services::HdfsNative>(iter)?.finish(),
             #[cfg(feature = "services-lakefs")]
             Scheme::Lakefs => Self::from_iter::<services::Lakefs>(iter)?.finish(),
-            #[cfg(feature = "services-nebula-graph")]
-            Scheme::NebulaGraph => Self::from_iter::<services::NebulaGraph>(iter)?.finish(),
             v => {
                 return Err(Error::new(
                     ErrorKind::Unsupported,
@@ -380,6 +370,11 @@ impl Operator {
 
     /// Create a new layer with dynamic dispatch.
     ///
+    /// Please note that `Layer` can modify internal contexts such as `HttpClient`
+    /// and `Runtime` for the operator. Therefore, it is recommended to add layers
+    /// before interacting with the storage. Adding or duplicating layers after
+    /// accessing the storage may result in unexpected behavior.
+    ///
     /// # Notes
     ///
     /// `OperatorBuilder::layer()` is using static dispatch which is zero
