@@ -24,7 +24,7 @@ use std::task::Poll;
 use futures::ready;
 use futures::Stream;
 
-use crate::raw::oio::Read;
+use crate::raw::oio::Read as _;
 use crate::raw::*;
 use crate::*;
 
@@ -88,7 +88,7 @@ impl ChunkedReader {
     /// We don't need to handle `Executor::timeout` since we are outside the layer.
     fn new(ctx: Arc<ReadContext>, range: BytesRange) -> Self {
         let tasks = ConcurrentTasks::new(
-            ctx.args().executor().cloned().unwrap_or_default(),
+            ctx.accessor().info().executor(),
             ctx.options().concurrent(),
             |mut r: oio::Reader| {
                 Box::pin(async {
@@ -125,16 +125,17 @@ impl oio::Read for ChunkedReader {
     }
 }
 
-/// BufferStream is a stream of buffers.
+/// BufferStream is a stream of buffers, created by [`Reader::into_stream`]
 ///
-/// # Notes
-///
-/// The underlying reader is either a StreamingReader or a ChunkedReader.
-///
-/// - If chunk is None, BufferStream will use StreamingReader to iterate
-///   data in streaming way.
-/// - Otherwise, BufferStream will use ChunkedReader to read data in chunks.
+/// `BufferStream` implements `Stream` trait.
 pub struct BufferStream {
+    /// # Notes to maintainers
+    ///
+    /// The underlying reader is either a StreamingReader or a ChunkedReader.
+    ///
+    /// - If chunk is None, BufferStream will use StreamingReader to iterate
+    ///   data in streaming way.
+    /// - Otherwise, BufferStream will use ChunkedReader to read data in chunks.
     state: State,
 }
 
@@ -145,7 +146,7 @@ enum State {
 
 impl BufferStream {
     /// Create a new buffer stream with already calculated offset and size.
-    pub fn new(ctx: Arc<ReadContext>, offset: u64, size: Option<u64>) -> Self {
+    pub(crate) fn new(ctx: Arc<ReadContext>, offset: u64, size: Option<u64>) -> Self {
         debug_assert!(
             size.is_some() || ctx.options().chunk().is_none(),
             "size must be known if chunk is set"
@@ -166,7 +167,10 @@ impl BufferStream {
     ///
     /// If users is going to perform chunked read but the read size is unknown, we will parse
     /// into range first.
-    pub async fn create(ctx: Arc<ReadContext>, range: impl RangeBounds<u64>) -> Result<Self> {
+    pub(crate) async fn create(
+        ctx: Arc<ReadContext>,
+        range: impl RangeBounds<u64>,
+    ) -> Result<Self> {
         let reader = if ctx.options().chunk().is_some() {
             let range = ctx.parse_into_range(range).await?;
             TwoWays::Two(ChunkedReader::new(ctx, range.into()))
